
# q-expansion of newform 640.2.a.e, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 640
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [1], [-2], [-6], [2], [-6], [2], [-6], [6], [-4], [6], [-2], [-4], [10], [2], [-10], [-10], [4], [-16], [-6], [0], [8], [6], [2], [-10], [18], [16], [-10], [-6], [18], [-10], [-22], [6], [14], [8], [-2], [8], [-10], [-18], [10], [-10], [-12], [10], [10], [20], [-10], [-10], [-28], [-10], [-14], [-16], [-14], [-6], [-6], [2], [30], [-20], [22], [-6], [20], [22], [-20], [-16], [-14], [-6], [10], [-22], [-8], [-18], [-6], [4], [-22], [-26], [-26], [-18], [-2], [10], [-2], [22], [26], [30], [12], [-14], [-4], [-36], [-30], [-6], [38], [22], [-12], [8], [-2], [-6], [2], [26], [-10], [-6], [28], [-34], [-28], [-2], [24], [6], [26], [34], [-8], [34], [-12], [-38], [18], [34], [34], [-34], [-8], [-30], [24], [-18], [-14], [10], [22], [-6], [-6], [-36], [30], [-18], [-10], [0], [-2], [30], [-30], [-6], [36], [22], [-6], [38], [-14], [4], [-54], [-22], [26], [22], [-14], [-8], [6], [-36], [34], [-30], [6], [6], [-34], [30], [-40], [14], [-24], [4], [-28], [18], [26], [14], [12], [-54], [6], [34], [10], [-30], [-52], [-6]]
